RectangleMorph subclass: #HangManAlphabetMorph
	instanceVariableNames: 'letters letterMorphs '
	classVariableNames: 'Alphabet '
	poolDictionaries: ''
	category: 'Daves-Hangman'!
!HangManAlphabetMorph commentStamp: '<historical>' prior: 0!
I hold an alphabet of instances of HangManLetterMorph and present them in a nicely aligned row. I and my submorphs allow dragging one of the submorphs to a target one time.

An alphabet depends on the language, of course, with English and French being the only supported languages at this point.!


!HangManAlphabetMorph methodsFor: 'dropping/grabbing' stamp: 'dns 1/8/2000 18:09'!
repelsMorph: aMorph event: ev

	^ (aMorph isKindOf: HangManLetterMorph) not! !


!HangManAlphabetMorph methodsFor: 'initialization' stamp: 'dns 1/8/2000 14:59'!
buildLetterMorphs
	| morph offset pos |
	letterMorphs := Array new: letters size.

	letters do: [ :ch |
		morph := HangManLetterMorph letter: ch.
		offset := self charOffset: ch.
		letterMorphs at: offset put: morph ].

	pos := self left + 2.
	letterMorphs do: [ :m |
		m topLeft: pos @ (self top + 2).
		pos := pos + m width + 1.
		self addMorph: m ].

	self width: pos - 2 + (self borderWidth * 2).
	self height: (letterMorphs at: (self charOffset: letters first)) height + (self borderWidth * 2)! !

!HangManAlphabetMorph methodsFor: 'initialization' stamp: 'dns 1/11/2000 20:16'!
charOffset: ch

	^ letters indexOf: ch ifAbsent: [ 0 ]! !

!HangManAlphabetMorph methodsFor: 'initialization' stamp: 'dns 1/10/2000 14:22'!
initialize

	super initialize.

	letters := Alphabet.
	self buildLetterMorphs.

	self borderWidth: 1.
	self borderColor: Color black.

	self color: Color white.

	self openDragAndDrop! !


!HangManAlphabetMorph methodsFor: 'accessing' stamp: 'dns 1/8/2000 20:15'!
reset
	" Reset the morph and its submorphs to their original state after being used "

	letterMorphs do: [ :aMorph | aMorph reset ].
	self changed! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

HangManAlphabetMorph class
	instanceVariableNames: ''!

!HangManAlphabetMorph class methodsFor: 'initialize-release' stamp: 'dns 1/10/2000 14:41'!
new

	Alphabet isNil ifTrue: [
		self english ].

	^ super new! !


!HangManAlphabetMorph class methodsFor: 'alphabets' stamp: 'dns 1/10/2000 14:40'!
english

	Alphabet := 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'! !

!HangManAlphabetMorph class methodsFor: 'alphabets' stamp: 'dns 1/12/2000 15:34'!
french
	" Just the non-accented  uppercase characters "

	Alphabet := 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'! !


RectangleMorph subclass: #HangManLetterMorph
	instanceVariableNames: 'letter string font position letterColor canDrag dragging '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Daves-Hangman'!
!HangManLetterMorph commentStamp: '<historical>' prior: 0!
I hold, and display, one letter in a hangman game. My instances go into an instance of HangManAlphabetMorph. 

My instances can be dragged and dropped anywhere, but only do something when the target is an instance of HangManPhraseMorph, which looks to see what letter my instance represents; afterwards my instance turns itself grey to show it's been used once and will not let itself be dragged again.!


!HangManLetterMorph methodsFor: 'drawing' stamp: 'dns 1/7/2000 20:29'!
drawOn: aCanvas

	super drawOn: aCanvas.

	aCanvas text: string bounds: (bounds insetBy: 1@1) font: font color: letterColor! !


!HangManLetterMorph methodsFor: 'accessing' stamp: 'dns 1/9/2000 15:23'!
balloonText

	^ 'The letter ', string, ';
drag it to the phrase box below
to see if it is in the phrase.'! !

!HangManLetterMorph methodsFor: 'accessing' stamp: 'dns 1/7/2000 15:49'!
letter

	^ letter! !

!HangManLetterMorph methodsFor: 'accessing' stamp: 'dns 1/7/2000 16:29'!
letter: aCharacter
	| form |

	letter := aCharacter.
	string := letter asString.
	form := font characterFormAt: letter.
	self width: form width + 2.
	self height: form height + 2! !

!HangManLetterMorph methodsFor: 'accessing' stamp: 'dns 1/7/2000 20:29'!
letterColor: aColor

	letterColor := aColor! !

!HangManLetterMorph methodsFor: 'accessing' stamp: 'dns 1/8/2000 15:18'!
reset
	" Reset the letter to its original state after possibly being used "

	self letterColor: Color black.
	canDrag := true.! !


!HangManLetterMorph methodsFor: 'initialization' stamp: 'dns 1/8/2000 17:59'!
initialize

	super initialize.

	self borderWidth: 1.
	self borderColor: Color veryLightGray.
	self color: Color white.
	self letterColor: Color black.
	font := StrikeFont familyName: #NewYork size: 36.
	canDrag := true.
	dragging := false.! !


!HangManLetterMorph methodsFor: 'printing' stamp: 'dns 1/7/2000 17:55'!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: ' ''', string, ''' '! !


!HangManLetterMorph methodsFor: 'dropping/grabbing' stamp: 'dns 1/8/2000 17:59'!
aboutToBeGrabbedBy: aHand

	canDrag ifFalse: [ ^ nil ].
	dragging := true.
	^ super aboutToBeGrabbedBy: aHand! !

!HangManLetterMorph methodsFor: 'dropping/grabbing' stamp: 'dns 1/8/2000 18:01'!
justDroppedInto: aMorph event: anEvent

	(aMorph isKindOf: HangManPhraseMorph) ifTrue: [
		self letterColor: Color lightGray.
		canDrag := false ].

	dragging ifTrue: [
		dragging := false.
		self slideBackToFormerSituation: anEvent. ]! !

!HangManLetterMorph methodsFor: 'dropping/grabbing' stamp: 'dns 1/10/2000 16:08'!
repelsMorph: aMorph event: ev

	^ (aMorph isKindOf: HangManLetterMorph) not! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

HangManLetterMorph class
	instanceVariableNames: ''!

!HangManLetterMorph class methodsFor: 'instance creation' stamp: 'dns 1/7/2000 15:48'!
letter: aCharacter

	^ self new
		letter: aCharacter! !


AlignmentMorph subclass: #HangManMorph
	instanceVariableNames: 'alphabetMorph phraseMorph buttonRowMorph scoreMorph initialPhrase openInWindow startButton '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Daves-Hangman'!
!HangManMorph commentStamp: '<historical>' prior: 0!
I present a hang man game and let it be played. I can be opened in the world or in a window.

Try these expressions to start a game:

   HangManMorph new openInWorld

   HangManMorph english new openInWorld

   HangManMorph french new openInWorld

   (HangManMorph initialPhrase: 'Squeak will take over the world, someday!!') openInWorld

To add other languages, see 

   * HangManAlphabetMorph class messages,
   * HangManPhrases class messages, and
   * HangMan class messages,

and follow the example for French. There is one new method to add to each class, named the same as the language name (italian, german, ...).!


!HangManMorph methodsFor: 'opening/closing' stamp: 'mir 1/18/2000 20:30'!
buildButtonRow
	| aRow aButton |

	aRow := AlignmentMorph new
		orientation: #horizontal;
		centering: #center;
		hResizing: #shrinkWrap;
		color: Color veryLightGray.

"	aButton := SimpleButtonMorph new
		label: 'Start Game';
		target: self;
		actionSelector: #buttonQuit.
	aRow addMorphBack: aButton.
	aButton visible: false.
	startButton _ aButton.
"
	aButton := SimpleButtonMorph new
		label: 'New Game';
		target: self;
		actionSelector: #buttonNewGame.
	aRow addMorphBack: aButton.

	^ aRow

! !

!HangManMorph methodsFor: 'opening/closing' stamp: 'dns 1/9/2000 15:55'!
buildLayout

	self orientation: #vertical.
	self centering: #center.

	self color: Color veryLightGray.
	self borderWidth: 1.

	alphabetMorph := HangManAlphabetMorph new.
	self addMorphFront: alphabetMorph.

	phraseMorph := HangManPhraseMorph phrase: self initialPhrase.
	phraseMorph wrongGuessTarget: self withSelector: #updateScore:.
	self addMorphBack: phraseMorph.

	buttonRowMorph := self buildButtonRow.
	self addMorphBack: buttonRowMorph.

	scoreMorph := self buildScore.
	self addMorphBack: scoreMorph! !

!HangManMorph methodsFor: 'opening/closing' stamp: 'dns 1/9/2000 15:45'!
buildScore

	^ LedMorph new
		digits: 2;
		extent: (2*10@15).! !

!HangManMorph methodsFor: 'opening/closing' stamp: 'dns 1/9/2000 15:12'!
openInWindow

	self buildLayout.
	openInWindow := true.
	super openInWindow! !

!HangManMorph methodsFor: 'opening/closing' stamp: 'dns 1/9/2000 15:12'!
openInWorld

	self buildLayout.
	openInWindow := false.
	super openInWorld! !


!HangManMorph methodsFor: 'initialization' stamp: 'dns 1/14/2000 19:05'!
initialize

	super initialize.
	openInWindow := false.
	self inset: 10! !


!HangManMorph methodsFor: 'accessing' stamp: 'dns 1/9/2000 15:08'!
initialPhrase
	| str |

	initialPhrase isNil ifFalse: [
		str := initialPhrase.
		initialPhrase := nil.
		^ str ].

	^ HangManPhrases pickOne! !

!HangManMorph methodsFor: 'accessing' stamp: 'dns 1/9/2000 14:41'!
initialPhrase: aString

	initialPhrase := aString! !


!HangManMorph methodsFor: 'actions' stamp: 'dns 1/9/2000 15:39'!
buttonNewGame

	alphabetMorph reset.
	phraseMorph phrase: HangManPhrases pickOne.
	phraseMorph reset.
	openInWindow ifTrue: [
		owner width: (phraseMorph width max: alphabetMorph width) + 
			(owner borderWidth * 2) + 8 ].
! !

!HangManMorph methodsFor: 'actions' stamp: 'dns 1/9/2000 15:14'!
buttonQuit

	openInWindow
		ifFalse: [ self delete ]
		ifTrue: [ self owner delete ]
! !

!HangManMorph methodsFor: 'actions' stamp: 'dns 1/9/2000 16:12'!
updateScore: anInteger

	scoreMorph value: anInteger! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

HangManMorph class
	instanceVariableNames: ''!

!HangManMorph class methodsFor: 'instance creation' stamp: 'dns 1/9/2000 15:22'!
initialPhrase: aString

	^ self new
		initialPhrase: aString! !

!HangManMorph class methodsFor: 'instance creation' stamp: 'mir 1/18/2000 20:38'!
oceanicPanic

	| hangMan |
	hangMan _ HangManMorph english new.
	hangMan
		initialPhrase: 'Follow that Squeak';
		beSticky;
		openInWorld.
	hangMan position: (hangMan fullBounds align: hangMan fullBounds center with: Display boundingBox center) origin.
	^hangMan
! !


!HangManMorph class methodsFor: 'language' stamp: 'dns 1/10/2000 14:41'!
english

	HangManAlphabetMorph english.
	HangManPhrases english.! !

!HangManMorph class methodsFor: 'language' stamp: 'dns 1/11/2000 20:12'!
french

	HangManAlphabetMorph french.
	HangManPhrases french.! !


RectangleMorph subclass: #HangManPhraseMorph
	instanceVariableNames: 'phrase displayPhrase guesses font scanner wrongGuesses wrongGuessSelector wrongGuessTarget '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Daves-Hangman'!
!HangManPhraseMorph commentStamp: '<historical>' prior: 0!
I hold one phrase and I display it in a rectangle morph with asterisks replacing any unguessed letters. Special characters and blanks are never hidden.

I handle accented characters: guess and 'a' and I also look for all accented forms. (The default font does not contain a character for the uppercase version of ''; ComicSans does but it is not as nice a font to use for default.)

When I'm opened by myself, I put a menu item in the halo menu so that you can make guesses of letters.!


!HangManPhraseMorph methodsFor: 'private' stamp: 'dns 1/9/2000 16:04'!
buildDisplayPhrase

	| ch |
	displayPhrase := String new: phrase size.
	displayPhrase atAllPut: $*.

	1 to: phrase size do: [ :n |
		ch := phrase at: n.
		(guesses includes: ch) ifTrue: [
			displayPhrase at: n put: ch ] ].

	self width: ((scanner stringWidth: phrase) max: (scanner stringWidth: displayPhrase)).
	self changed
				! !


!HangManPhraseMorph methodsFor: 'accessing' stamp: 'dns 1/11/2000 20:28'!
guess: aCharacter

	(guesses includes: aCharacter) ifTrue: [ ^ false ].

	(phrase includes: aCharacter asLowercase) | 
	(phrase includes: aCharacter asUppercase) ifTrue: [
		guesses := guesses, 
			aCharacter asLowercase asString, 
			aCharacter asUppercase asString,
			(self specialFormsOf: aCharacter asLowercase).
		self buildDisplayPhrase.
		^ true ].
	wrongGuesses := wrongGuesses + 1.
	self notifyWrongGuessTarget.
	^ false! !

!HangManPhraseMorph methodsFor: 'accessing' stamp: 'dns 1/9/2000 16:05'!
phrase: aString

	phrase := aString.

	self buildDisplayPhrase! !

!HangManPhraseMorph methodsFor: 'accessing' stamp: 'dns 1/9/2000 16:07'!
reset

	guesses := self class initialGuesses.
	wrongGuesses := 0.
	self notifyWrongGuessTarget.
	self buildDisplayPhrase.! !

!HangManPhraseMorph methodsFor: 'accessing' stamp: 'dns 1/17/2000 14:27'!
specialFormsOf: ch
	" Given a lowercase unaccented letter, answer all lowercase and uppercase accented forms of that character. "

"!!#$%&''()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~"

	ch == $a ifTrue: [ ^ '̇' ].
	ch == $c ifTrue: [ ^ '' ].
	ch == $e ifTrue: [ ^ '' ].
	ch == $i ifTrue: [ ^ '' ].
	ch == $n ifTrue: [ ^ '' ].
	ch == $o ifTrue: [ ^ '͗' ].
	ch == $u ifTrue: [ ^ '' ].
	ch == $y ifTrue: [ ^ '' ]. "The first char is a cap Y with two dots in some fonts"
	^ ''! !


!HangManPhraseMorph methodsFor: 'menu' stamp: 'dns 1/10/2000 16:12'!
addCustomMenuItems: aCustomMenu hand: aHandMorph

	super addCustomMenuItems: aCustomMenu hand: aHandMorph.

	" If morph is opened by itself, add a menu for guessing letters "
	(owner isKindOf: HangManMorph) ifFalse: [
		aCustomMenu addLine.
		aCustomMenu addList: 
			#(	('guess a letter' putUpGuessMenu:)) ]! !

!HangManPhraseMorph methodsFor: 'menu' stamp: 'dns 1/10/2000 16:13'!
putUpGuessMenu: event
	| aMenu items |

	aMenu := MenuMorph new defaultTarget: self.
	aMenu addTitle: 'Guess one letter'.
	items := 'abcdefghijklmnopqrstuvwxyz' asOrderedCollection.
	items removeAllFoundIn: guesses.
	items do: [ :ch |
		aMenu add: ch asString selector: #guess: argument: ch ].

	aMenu popUpAt: event cursorPoint forHand: event hand ! !


!HangManPhraseMorph methodsFor: 'initialization' stamp: 'dns 1/9/2000 15:47'!
initialize

	super initialize.

	self color: Color white.
	self borderWidth: 1.
	self height: 40.

	font := StrikeFont familyName: #DefaultTextStyle size: 36.
	scanner := QuickPrint newOn: Display box: Display boundingBox font: font.

	displayPhrase := 'Empty'.
	guesses := self class initialGuesses.
	wrongGuesses := 0.

	self openDragAndDrop! !


!HangManPhraseMorph methodsFor: 'drawing' stamp: 'dns 1/11/2000 19:31'!
drawOn: aCanvas

	super drawOn: aCanvas.

	aCanvas
		text: displayPhrase
		bounds: ((bounds insetBy: 1@1) translateBy: (0@7))
		font: font
		color: Color black.! !


!HangManPhraseMorph methodsFor: 'dropping/grabbing' stamp: 'dns 1/8/2000 17:36'!
acceptDroppingMorph: aHangManLetter event: evt

	self guess: aHangManLetter letter! !

!HangManPhraseMorph methodsFor: 'dropping/grabbing' stamp: 'dns 1/8/2000 14:59'!
repelsMorph: aMorph event: ev

	^ (aMorph isKindOf: HangManLetterMorph) not! !


!HangManPhraseMorph methodsFor: 'notifying' stamp: 'dns 1/9/2000 15:53'!
notifyWrongGuessTarget

	wrongGuessSelector notNil ifTrue: [
		wrongGuessTarget perform: wrongGuessSelector with: wrongGuesses ]! !

!HangManPhraseMorph methodsFor: 'notifying' stamp: 'dns 1/9/2000 15:54'!
wrongGuessTarget: anObject withSelector: aSelector

	wrongGuessTarget := anObject.
	wrongGuessSelector := aSelector! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

HangManPhraseMorph class
	instanceVariableNames: ''!

!HangManPhraseMorph class methodsFor: 'private' stamp: 'dns 1/8/2000 20:12'!
initialGuesses

	^ ' .:;'' -,!!@#$%&*()"?/`0123456789'! !


!HangManPhraseMorph class methodsFor: 'instance creation' stamp: 'dns 1/7/2000 18:34'!
phrase: aString

	^ self new
		phrase: aString! !


Object subclass: #HangManPhrases
	instanceVariableNames: ''
	classVariableNames: 'Phrases '
	poolDictionaries: ''
	category: 'Daves-Hangman'!
!HangManPhrases commentStamp: '<historical>' prior: 0!
I hold the phrases for a hangman game. I can hold phrases for multiple languages but right now I'm only set up for English, and French (three phrases only).!


"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

HangManPhrases class
	instanceVariableNames: ''!

!HangManPhrases class methodsFor: 'phrases' stamp: 'dns 1/10/2000 15:57'!
english

	Phrases := #(
'Squeak will take over the world.'			"Prediction"
'Of Mice and Men'							"Book title"
'Moby Dick'									"Book title"
'An Instance of the Fingerpost'				"Ian Pear, book title"
'Blood, toil, tears, and sweat'				"Winston Churchill"
'It ain''t over ''til it''s over.'				"Yogi Berra"
'Deja Vu all over again'						"Yogi Berra"
'He who laughs last thinks slowest.'			"Unknown"
'Huckleberry Finn'							"Book title"
'Man shall not live by bread alone.'			"Bible"
'Make everything as simple as possible, but not simpler.'			"Albert Einstein"
'A billion here, a billion there, pretty soon it''s real money.'		"Sen. Everett Dirksen"
'It is the theory that decides what can be observed.'				"Albert Einstein"
'The eye sees only what the mind is prepared to comprehend.'	"Robertson Davies"
'People only see what they are prepared to see.'					"Ralph Waldo Emerson"
'Seek simplicity, and distrust it.'  			"Alfred North Whitehead" 
'Belief gets in the way of learning.'  		"Robert Heinlein, Time Enough for Love" 
'A prudent question is one half of wisdom.'	"Francis Bacon" 
'Life is serious, but art is fun.'  			"John Irving, The Hotel New Hampshire" 
'All poets are mad.'  						"Robert Burton, The Anatomy of Melancholy"
'Of all lies, art is the least untrue.'  		"Gustave Flaubert" 
'Art is a lie that makes us realize the truth.'	"Pablo Picasso"
'Less is more.'								"Robert Browning, 'Andrea del Sarto'"
'The body says what words cannot.'			"Martha Graham" 
'Never underestimate the power of human stupidity.'  "Robert Heinlein, Time Enough for Love"
'Talk Small and carry a big class library'	"Gordon Mohr, XEROX PARC"
'Heisenberg might have slept here.'			"Aaron Avery, University of Wisconsin"
'When in doubt, tell the truth.'				"Mark Twain"
'The Decline and Fall of the Roman Empire'	"Gibbons book title"
'Edward Kennedy Ellington'					"Composer, performer"
'It Don''t Mean a Thing if it Ain''t Got That Swing'	"Song Title: Duke Ellington"
'Take the A-Train'							"Song title: Billy Strayhorn"
'Sophisticated Lady'							"Song title: Duke Ellington"
'Porgy and Bess'							"Opera title: George Gershwin"
'Mood Indigo'								"Song title: Albany Bigard, Duke Ellington"
'floating-point number'						"Computer term"
'central processing unit'						"Computer term"
'Count Basie'								"Jazz player, leader, composer"
	)

	! !

!HangManPhrases class methodsFor: 'phrases' stamp: 'dns 1/10/2000 15:41'!
pickOne
	" HangManPhrases pickOne "

	Phrases isNil ifTrue: [
		Phrases := self english ].

	^ Phrases at: Phrases size atRandom
	! !
